class Error(Exception):
    ...


class AuthError(Error):
    ...


class TypeMethodError(Error):
    ...


class TypeAnti(Error):
    ...


class ErrorServer(Error):
    ...


class ErrorMethod(Error):
    ...


class ErrorPrivatyKey(Error):
    ...


class NOT_REGISTERED(Error):
    ...


class TOO_REQUESTS(Error):
    ...
